<?php

namespace FM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use FM\Factory\FmIntegracaoHcmFactory;
use FM\Service\FuncionariosHierarqService;

class FmIntegracaoHcmService
{
    protected $Environment;
    protected $newCode;
    protected $IntegracaoHcmFactory;
    protected $connection;
    protected $FindService;
    protected $FuncionariosHierarqService;

    public function __construct(Environment $Environment, NewCode $newCode, EntityManager $EntityManager, FmIntegracaoHcmFactory $IntegracaoHcmFactory, FindService $FindService, FuncionariosHierarqService $FuncionariosHierarqService)
    {
        $this->connection                   = $EntityManager->getConnection();
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->IntegracaoHcmFactory         = $IntegracaoHcmFactory;
        $this->FindService                  = $FindService;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
    }

    public function getTarefasMapaOperacao()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);

        $params = array(
            "CDOPERADOR" => $cdoperador
        );

        return $this->IntegracaoHcmFactory->getTarefasMapaOperacao($params, $listaOperadores);
    }

    public function getOperadorMapaOperacoes($parametros)
    {
        $dadosIntegracao = $this->integracaoHcm();
        list($token, $url, $secret) = $dadosIntegracao;

        $operadores = [];

        if ($parametros['IDTAREFAEQUIPE'] == 'S') {
            $operadores = $this->IntegracaoHcmFactory->getOperadoresDaEquipe([
                "NREQUIPE" => $parametros['NREQUIPE'],
                "NRCHECKLISTAGEND" => $parametros['NRCHECKLISTAGEND']
            ]);
        }

        if (empty($operadores)) {
            // Caso não sejam operadores de equipe, buscar diretamente pelo operador
            $inscricao = $this->buscarInscricaoParceiro($parametros['CDOPERADOR']);

            $operadores = [
                "data" => [
                    [
                        "CDOPERADOR"        => $parametros['CDOPERADOR'],
                        "NMOPERADOR"        => $parametros['NMOPERADOR'],
                        "NRINSCRICAOPARC"   => $inscricao,
                        "RNUM"              => 1,
                        "TOTAL_LINES"       => 1
                    ]
                ],
                "pagination" => [
                    "total" => 1, // Total de itens
                    "page"  => 1, // Página atual
                    "limit" => 10 // Limite de itens por página
                ]
            ];
        } else {
            // Atualizar os operadores com NRINSCRICAOPARC válido
            foreach ($operadores['data'] as &$operador) {
                $inscricao = $this->buscarInscricaoParceiro($operador['CDOPERADOR']);
                $operador['NRINSCRICAOPARC'] = $inscricao;
            }
        }

        foreach ($operadores['data'] as &$operador) {
            if ($operador['NRINSCRICAOPARC'] !== null) {
                $dadosPonto = $this->getMarcacaoPontoEspelho($url, $token, $secret, $parametros, $operador['NRINSCRICAOPARC']);
                $operador['DADOSMARCACOESPONTO'] = $dadosPonto['dataset']['getMarcacaoPontoEspelho'][0];
                
                $dadosPonto = $this->getFerias($url, $token, $secret, $parametros, $operador['NRINSCRICAOPARC']);
                $operador['DADOSAFERIAS'] = $dadosPonto['dataset']['ferias'];

                $dadosPonto = $this->getAlteracoesEscala($url, $token, $secret, $parametros, $operador['NRINSCRICAOPARC']);
                $operador['DADOSESCALA'] = $dadosPonto['dataset'];

                $dadosPonto = $this->getAfastamentos($url, $token, $secret, $parametros, $operador['NRINSCRICAOPARC']);
                $operador['DADOSAFASTAMENTO'] = $dadosPonto;
            }
        }

        return $operadores;
    }

    /**
     * Busca e valida a inscrição do parceiro (NRINSCRICAOPARC).
     *
     * @param string $cdOperador
     * @return string|null
     */
    private function buscarInscricaoParceiro($cdOperador)
    {
        $inscricaoOper = $this->IntegracaoHcmFactory->getInscricaoParceiro([
            "CDOPERADOR" => $cdOperador,
        ]);

        if (empty($inscricaoOper['data'])) {
            return null;
        }

        $nrInscricaoOper = $inscricaoOper['data'][0]['NRINSCRICAOPARC'];

        return $this->validarCPF($nrInscricaoOper) ? $nrInscricaoOper : null;
    }

    public function validarCPF($cpf) {
        // Remove caracteres não numéricos
        $cpf = preg_replace('/[^0-9]/', '', $cpf);
    
        // Verifica se o CPF tem 11 dígitos
        if (strlen($cpf) != 11) {
            return false;
        }
    
        // Verifica se todos os dígitos são iguais (ex.: 111.111.111-11 é inválido)
        if (preg_match('/^(\d)\1{10}$/', $cpf)) {
            return false;
        }
    
        // Calcula os dígitos verificadores
        for ($t = 9; $t < 11; $t++) {
            $d = 0;
            for ($c = 0; $c < $t; $c++) {
                $d += $cpf[$c] * (($t + 1) - $c);
            }
            $d = ((10 * $d) % 11) % 10;
            if ($cpf[$c] != $d) {
                return false;
            }
        }
    
        return true;
    }

    public function integracaoHcm()
    {
        $token = '';
        
        $paramGeral = $this->IntegracaoHcmFactory->getParamsGeral();
        $url = 'https://hcmservices.teknisa.com/index.php'; // $paramGeral[0]['DSURLHCM'];
        $secret = $paramGeral[0]['DSSECRETHCM'];
    
        $access = $this->checkAccess($url, $paramGeral[0]['DSTOKENHCM'], $secret);
    
        if($access == 'Denied') {
            $this->connection->beginTransaction();
            $token = $this->requestToken($url, $paramGeral[0]['DSIDHCM'], $secret);
            $params = array(
                "TOKEN" => $token
            );
            $this->IntegracaoHcmFactory->updateToken($params);
            $this->connection->commit();
        } else {
            $token = $paramGeral[0]['DSTOKENHCM'];
        }

        return [$token, $url, $secret];
    }

    public function getMarcacaoPontoEspelho($uri, $token, $secret, $parametros, $nrInscricaoOper)
    {
        $uri_produ = $uri."/getMarcacaoPontoEspelho";
        $header = array(
            "OAuth-Token: $token",
            "OAuth-SecretId: $secret"
        );
        $header[] = 'Content-Type: application/json';
        $data = array(
            'requestType' => 'Row',
                'row' => array(
                    "DTINICIO"          => $parametros['DTHORAAGEND'],
                    "DTFIM"             => $parametros['DTHRFINALAGEND'],
                    "NRCPFPESSOA"       => $nrInscricaoOper
                ));

        return $this->connectionWebService($header, $uri_produ, $data);
    }

    public function getAfastamentos($uri, $token, $secret, $parametros, $nrInscricaoOper)
    {
        $uri_produ = $uri."/getAfastamentos";
        $header = array(
            "OAuth-Token: $token",
            "OAuth-SecretId: $secret"
        );
        $header[] = 'Content-Type: application/json';
        $data = array(
            'requestType' => 'Row',
                'row' => array(
                    "DTINICIO"          => $parametros['DTHORAAGEND'],
                    "DTFIM"             => $parametros['DTHRFINALAGEND'],
                    "NRCPFPESSOA"       => $nrInscricaoOper
                ));

        return $this->connectionWebService($header, $uri_produ, $data);
    }

    public function getAlteracoesEscala($uri, $token, $secret, $parametros, $nrInscricaoOper)
    {
        $uri_produ = $uri."/getAlteracoesEscala";
        $header = array(
            "OAuth-Token: $token",
            "OAuth-SecretId: $secret"
        );
        $header[] = 'Content-Type: application/json';
        $data = array(
            'requestType' => 'Row',
                'row' => array(
                    "DTINICIO"          => $parametros['DTHORAAGEND'],
                    "NRCPFPESSOA"       => $nrInscricaoOper
                ));

        return $this->connectionWebService($header, $uri_produ, $data);
    }

    public function getFerias($uri, $token, $secret, $parametros, $nrInscricaoOper)
    {
        $uri_produ = $uri."/getFerias";
        $header = array(
            "OAuth-Token: $token",
            "OAuth-SecretId: $secret"
        );
        $header[] = 'Content-Type: application/json';
        $data = array(
            'requestType' => 'Row',
                'row' => array(
                    "DTINICIO"          => $parametros['DTHORAAGEND'],
                    "NRCPFPESSOA"       => $nrInscricaoOper
                ));

        return $this->connectionWebService($header, $uri_produ, $data);
    }

    public function requestToken ($uri, $id, $secret) 
    {
        $uri_produ = $uri.'/requestToken';
        $clientData = array(
            "CLIENT_ID" => $id,
            "CLIENT_SECRET"=> $secret,
        );
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($clientData));
        $result = $this->connectionWebService($header, $uri_produ, $clientData);
        return $result['dataset']['token'][0];
    }

    public function checkAccess($uri, $token, $secret)
    {
        $uri_produ = $uri.'/checkAccess';
        $clientData = array(
            "TOKEN" => $token,
            "CLIENT_SECRET" => $secret,
        );

        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($clientData));

        $result = $this->connectionWebService($header, $uri_produ, $clientData);

        if ($result['messages'][0]['message'] == "Authorized") {
            return "Authorized";
        } else {
            return "Denied";
        }
    }

    public function connectionWebService($header, $uri_produ, $data) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $uri_produ);
        curl_setopt($ch, CURLOPT_POST, count($data));
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        $result = curl_exec($ch);
        curl_close($ch);
        return json_decode($result, true);
    }

}
